///////////////////////////////////////////////////////////////////////////////////////
// LICENSE INFO
// ------------------------------------------------------------------------------------
// WE WILL ONLY DISTRIBUTE THIS HEADER AND THE STATICALLY LINKED LIB, ALONG WITH A DEMO
///////////////////////////////////////////////////////////////////////////////////////
#ifndef _GLU_KERNEL_
#define _GLU_KERNEL_

#include <iostream>

namespace GLU {

class Kernel {

public:

    /**************************************************************/
    /* Constructors                                               */
    /**************************************************************/

    /* With text stream, for testing use */
    static Kernel *Create (
            std::istream &in,
            std::ostream &out,
            std::ostream &err
            );

    /* With raw data, more practical and conventional */
    static Kernel *Create (
            int n,      /* matrix size */
            int *ia,    /* row index */
            int *ja,    /* column index */
            float *a,   /* matrix A pointer */
            int nnz,    /* number of non-zeros */
            float *b,   /* vector b pointer */
            float *x,   /* unknowns x pointer */
            std::ostream &out,  /* output stream */
            std::ostream &err); /* error stream */

    virtual ~Kernel() {};

    /**************************************************************/
    /* Kernel interface                                           */
    /**************************************************************/
    enum CMD {
        PARSE, FILLIN, PERMUTATION, SYMBOLIC, LEVEL,
        PREDICT, GPUFACT, SOLVE, CLEAN };

    virtual void Do(CMD cmd) = 0;

    virtual int &nThreads() = 0;
    virtual int nThreads() const = 0;

    /**************************************************************/
    /* Internal data proxies, allowing user to change them        */
    /**************************************************************/
    virtual int &mtxDim() = 0;
    virtual int mtxDim() const = 0;

    virtual int *&rowIndex() = 0;
    virtual int *rowIndex() const = 0;

    virtual int *&colIndex() = 0;
    virtual int *colIndex() const = 0;

    virtual float *&mtxData() = 0;
    virtual float *mtxData() const = 0;

    virtual int &nnz() = 0;
    virtual int nnz() const = 0;

    virtual float *&bVec() = 0;
    virtual float *bVec() const = 0;

    virtual float *&xVec() = 0;
    virtual float *xVec() const = 0;

    /**************************************************************/
    /* Out/Err streams                                            */
    /**************************************************************/
    virtual std::ostream &outStream() = 0;
    virtual std::ostream &errStream() = 0;

}; /* class Kernel */

}; /* namespace GLU */

#endif /* _GLU_KERNEL_ */
