#ifndef _GLU_KERNEL_IMPL_H_
#define _GLU_KERNEL_IMPL_H_

#include <iostream>
#include "glu_kernel.h"
#include "matrix.h"
#include "symbolic.h"

namespace GLU {
class KernelImpl: public Kernel
{
public:

    KernelImpl(std::istream &in,
               std::ostream &out,
               std::ostream &err);

    KernelImpl(int n,      /* matrix size */
               int *ia,    /* row index */
               int *ja,    /* column index */
               float *a,   /* matrix A pointer */
               int nnz,    /* number of non-zeros */
               float *b,   /* vector b pointer */
               float *x,   /* unknowns x pointer */
               std::ostream &out,  /* output stream */
               std::ostream &err); /* error stream */

    virtual void Do(CMD cmd);
    virtual ~KernelImpl();

    /* inline proxies */
    virtual int &mtxDim() { return m_n; }
    virtual int mtxDim() const { return m_n; }

    virtual int *&rowIndex() { return m_ia; }
    virtual int *rowIndex() const { return m_ia; }

    virtual int *&colIndex() { return m_ja; }
    virtual int *colIndex() const { return m_ja; }

    virtual float *&mtxData() { return m_a; }
    virtual float *mtxData() const { return m_a; }

    virtual int &nnz() { return m_nnz; }
    virtual int nnz() const { return m_nnz; }

    virtual float *&bVec() { return m_b; }
    virtual float *bVec() const { return m_b; }

    virtual float *&xVec() { return m_x; }
    virtual float *xVec() const { return m_x; }

    virtual int &nThreads() { return m_nthreads; }
    virtual int nThreads() const { return m_nthreads; }

    virtual std::ostream &outStream() { return m_out; }
    virtual std::ostream &errStream() { return m_err; }

private:

    /* data */
    int m_n;        /* matrix size */
    int *m_ia;      /* row index */
    int *m_ja;      /* column index */
    float *m_a;     /* matrix A pointer */
    int m_nnz;      /* number of non-zeros */
    float *m_b;     /* vector b pointer */
    float *m_x;     /* unknowns x pointer */

    /* streams */
    std::istream &m_in;     /* input stream */
    std::ostream &m_out;    /* output stream */
    std::ostream &m_err;    /* error stream */

    /* ENCAPSULATED DATA/OBJECTS */
    int *m_permutation;
    Matrix m_original;
    Matrix m_permutated;
    Symbolic_Matrix m_symbolic;

    /* environment */
    int m_nthreads;

    /* worker functions */
    void do_parse();
    void do_fill_in();
    void do_permutation();
    void do_symbolic();
    void do_level();
    void do_predict();
    void do_gpu_fact();
    void do_solve();
    void do_clean();

}; /* class KernelImpl */

}; /* namespace GLU */

#endif /* _GLU_KERNEL_IMPL_H_ */
