#include <iostream>
#include <vector>
#include <set>
#include <iomanip>
#include <cstring>
#include <cmath>
#include <fstream>
#include "matrix.h"
#include "symbolic.h"
#include "numeric.h"
#include "Timer.h"
#include "preprocess.h"
#include "nicslu.h"

using namespace std;

void help_message()
{
    cout << endl;
    cout << "GLU  program V2.0" << endl;
    cout << "Usage: ./lu_cmd -i inputfile" << endl;
    cout << "Additional usage: ./lu_cmd -i inputfile -p" << endl;
    cout << "-p to enable perturbation" << endl;
}

int main(int argc, char** argv)
{
	SNicsLU *nicslu;
    
    Timer t;
    double utime;

    char *matrixName;
    bool perturb = false;
    bool printout = false;


    float *ax;
    unsigned int *ai, *ap; 
    int n, nnz;

    unsigned int *rp, *cp;
    int *piv;
    float *rows, *cols, *cscale;

    if (argc < 3) {
        help_message();
        return -1;
    }

    for (int i = 1; i < argc;) {
        if (strcmp(argv[i], "-i") == 0) {
            if(i+1 > argc) {
                help_message();
                return -1;
            }       
            matrixName = argv[i+1];
            i += 2;
        }
        else if (strcmp(argv[i], "-p") == 0) {
            perturb = true;
            i += 1;
        }        
        else if (strcmp(argv[i], "-pr") == 0) {
            printout = true;
            i += 1;
        }
        else {
            help_message();
            return -1;
        }
    }

    ax = NULL;
    ai = NULL;
    ap = NULL;

    rp = NULL;
    cp = NULL;
    piv = NULL;
    rows = NULL;
    cols = NULL;
    cscale = NULL; 

    nicslu = (SNicsLU *)malloc(sizeof(SNicsLU));

    int err = preprocess(matrixName, nicslu, &ax, &ai, &ap, rp, cp, piv, rows, cols, cscale);
    if (err)  
    {
    	// cout << "Reading matrix error" << endl;
    	exit(1);
    }

    n = (int) nicslu->n;
    nnz = (int) nicslu->nnz;

    cout << "Matrix Row: " << n << endl;
    // cout << "Matrix non-zero no.(nz): " << nnz << endl;

    Matrix A(cout, cerr, n, nnz);
    memcpy(&(A.x[0]), ax, sizeof(float) * nnz);
    memcpy(&(A.row_idx[0]), ai, sizeof(int) * nnz);
    memcpy(&(A.col_ptr[0]), ap, sizeof(int) * (n+1));
    cout << "Csc finished..." << endl;

    t.start();

    Symbolic_Matrix A_sym(cout, cerr);
    A_sym.symbolic(A);
    cout << "Symbolic finished... " << endl;

    A_sym.csr();
    cout << "CSR finished... " << endl;

    A_sym.predictLU(A);
    cout << "PredictLU... " << endl;
    
    t.elapsedUserTime(utime);
    t.elapsedWallclockTime(utime);
    cout << "Symbolic time " << utime << " ms" << endl;

    t.start();

    A_sym.U_W_leveling();
    cout << "U_W_leveling... " << endl;

    t.elapsedUserTime(utime);
    t.elapsedWallclockTime(utime); 
    cout << "Leveling time " << utime << " ms" << endl;

    cout << "\nFirst factorization... " << endl;  
    LUonDevice(A_sym, A, 1024, cout, cerr, perturb, printout); 
    
    cout << "\nSecond factorization with perturbation... " << endl; 
    A_sym.restore();
    LUonDevice(A_sym, A, 1024, cout, cerr, 1, printout);
}
