#ifndef _MATRIX_H_
#define _MATRIX_H_
#include <vector>
#include <string>

class Element
{
public:
	int row_idx;
	int col_idx;
	float val;
	Element(int r, int c, float x) : row_idx(r), col_idx(c), val(x) {}
};

class Matrix
{
public:
	int nnz; //# of nonzero
	int row_num; //# of row
	int col_num; //# of col
	std::vector<Element> cooInfo; //size nnz, element information in coo format
	std::vector<int> row_idx; // size nnz, row index of nonzero, for csc
	std::vector<int> col_ptr; // size col_num + 1, col pointer of nonzero, for csc
	std::vector<float> x; // size nnz, value of nonzero in csc
	std::vector<int> col_idx; // *** size nnz, col index of nonzero, for csr
	std::vector<int> row_ptr; // *** size row_num + 1, row pointer of nonzero, for csr
	
	void readCoo(char* filename);
	void readCoo(std::string filename);
	void readCooFromStream(std::istream &);
	void printCoo();
	void Coo2Csc_Csr();
	void printCsc();
	void printCsr();

    Matrix(std::ostream &out, std::ostream &err): m_out(out), m_err(err) {}
    Matrix(std::ostream &out, std::ostream &err, int n, int nnz): m_out(out), \
    m_err(err), row_idx(nnz), col_ptr(n+1), x(nnz), nnz(nnz), row_num(n), col_num(n) {}

private:
	bool csc_valid;
	bool csr_valid;

    std::ostream &m_out;
    std::ostream &m_err;
};

class Entry
{
public:
	int row_idx;
	float val;
	bool operator < (const Entry & e) const
	{
		return (row_idx < e.row_idx);
	}
	Entry(int r, float x) : row_idx(r), val(x) {}
};

#endif
