#include <iostream>
#include <fstream>
#include <iomanip>
#include <algorithm>
#include <string>
#include "matrix.h"

using namespace std;

bool cooComp (Element a, Element b)
{
	return (a.row_idx < b.row_idx) || ((a.row_idx == b.row_idx) && (a.col_idx < b.col_idx));
}

void Matrix :: readCoo(char* filename)
{
	ifstream fin;

	fin.open(filename);
	
	if (fin.fail())
	{
		cerr << "Cannot open \"" << filename << "\"!" << endl;
        exit(1);
	}

	fin >> row_num >> col_num >> nnz;

	int r, c;
	float x;
	for (int i = 0; i < nnz; ++i)
	{
		fin >> r >> c >> x;
		Element tmp(r-1, c-1, x);
		cooInfo.push_back(tmp);
	}
}

void Matrix :: readCoo(string filename)
{
	ifstream fin;

	fin.open(filename.c_str());
	
	if (fin.fail())
	{
		cerr << "Cannot open \"" << filename << "\"!" << endl;
        exit(1);
	}

	fin >> row_num >> col_num >> nnz;

	int r, c;
	float x;
	for (int i = 0; i < nnz; ++i)
	{
		fin >> r >> c >> x;
		Element tmp(r-1, c-1, x);
		cooInfo.push_back(tmp);
	}
}

void Matrix :: readCooFromStream(istream &is)
{
	is >> row_num >> col_num >> nnz;

	int r, c;
	float x;
	for (int i = 0; i < nnz; ++i)
	{
		is >> r >> c >> x;
		Element tmp(r-1, c-1, x);
		cooInfo.push_back(tmp);
	}
}

void Matrix :: Coo2Csc_Csr()
{
	//generate csc row_idx, col_ptr and x_csc
	vector<int> nz_col(col_num, 0);
	vector<int> nz_row(row_num, 0);
	for (int i = 0; i < cooInfo.size(); ++i)
	{
		row_idx.push_back(cooInfo[i].row_idx);
		x.push_back(cooInfo[i].val);
		nz_col[cooInfo[i].col_idx]++;
	}
	col_ptr.push_back(0);
	for (int i = 1; i < row_num; ++i)
	{
		col_ptr.push_back(col_ptr.back()+nz_col[i-1]);
	}
	col_ptr.push_back(nnz);
	m_out << "Csc finished..." << endl;
	//generate csr col_idx, row_ptr, and x_csr
	sort(cooInfo.begin(),cooInfo.end(), cooComp);
	for (int i = 0; i < cooInfo.size(); ++i)
	{
		col_idx.push_back(cooInfo[i].col_idx);
		nz_row[cooInfo[i].row_idx]++;
	}

	row_ptr.push_back(0);
	for (int i = 1; i < row_num; ++i)
	{
		int tmp = row_ptr.back() + nz_row[i-1];
		row_ptr.push_back(tmp);
	}
	row_ptr.push_back(nnz);
	m_out << "Csr finished..." << endl;
	csc_valid = true;
	csc_valid = true;
}

void Matrix :: printCoo()
{
	if (cooInfo.size() == 0)
	{
		m_out << "Empty Matrix" << endl;
		return;
	}

	m_out << "Row_num: " << row_num << endl;
	m_out << "Col_num: " << col_num << endl;
	m_out << "Nonzero num:" << nnz << endl;
	for (int i = 0; i < cooInfo.size(); ++i)
	{
		m_out << left << setw(10) << cooInfo[i].row_idx
		    << left << setw(10) << cooInfo[i].col_idx
			<< left << setw(10) << cooInfo[i].val << endl;
	}
}

void Matrix :: printCsc()
{
	m_out << "Row_num: " << row_num << endl;
	m_out << "Col_num: " << col_num << endl;
	m_out << "Nonzero num:" << nnz << endl;
	
	m_out << "Col_ptr: ";
	for (int i = 0; i <= col_num; ++i)
	{
		m_out << left << setw(8) << col_ptr[i];
	}
	m_out << endl;
	m_out << "Row_idx and value: " << endl;
	for (int i = 0; i < nnz; ++i)
	{
		m_out << left << setw(8) << row_idx[i] << left << setw(8) << x[i] << endl;
	}
}

void Matrix :: printCsr()
{
	if (!csr_valid)
	{
		m_out << "Empty Matrix" << endl;
		return;
	}

	m_out << "Row_num: " << row_num << endl;
	m_out << "Col_num: " << col_num << endl;
	m_out << "Nonzero num:" << nnz << endl;
	
	m_out << "Row_ptr: ";
	for (int i = 0; i <= row_num; ++i)
	{
		m_out << left << setw(8) << row_ptr[i];
	}
	
	m_out << endl;
	m_out << "Col_idx and value: " << endl;

	for (int i = 0; i < nnz; ++i)
	{
		m_out << left << setw(8) << col_idx[i] << endl;
	}
}
