#include <iostream>
#include <algorithm>
#include <cmath>
#include "matrix.h"

using namespace std;

// Permutation with dynamic  pivoting 
void permutation_pivot(Matrix &A, int* P, Matrix &A_new)
{
	int n = A.row_num;
	int nz = A.nnz;
	A_new.row_num = n;
	A_new.col_num = n;
	A_new.nnz = nz;

	int* Pinv = new int[n];
	for (int i = 0; i < n; ++i)
	{
		Pinv[P[i]] = i;
	}

	// Pinv [i] = k if i is kth pivot row
	for (int i = 0; i < n; ++i) 
	{
		double abs_pivot = 0; //pivot val
		int ppivrow = i; //pivot row
		int pdiag = i; //diag row
		// search current column i
		for (int p = A.col_ptr[i]; p < A.col_ptr[i+1]; ++p) 
		{
			int row = A.row_idx[p];	
			double xabs = fabs(A.x[p]);
			if (row < i) {
				continue;// only search lower matrix
			} else if (xabs > abs_pivot) { // find pivoting choice
				abs_pivot = xabs;
				ppivrow = row;
			}
		}
		// swap pivoting row and diag row
		int tmp = Pinv[pdiag];	
		Pinv[pdiag] = Pinv[ppivrow];
		Pinv[ppivrow] = tmp;
	}

	int jnew = 0, inew = 0;
	A_new.col_ptr.push_back(0);

	for (jnew = 0; jnew < n; ++jnew)
	{
		int j = P[jnew];
		A_new.col_ptr.push_back(A_new.col_ptr[jnew] + A.col_ptr[j+1] - A.col_ptr[j]);

		for (int i = A.col_ptr[j]; i < A.col_ptr[j+1]; ++i)
		{
			inew = Pinv[A.row_idx[i]];
			A_new.row_idx.push_back(inew);
			A_new.x.push_back(A.x[i]);
		}
	}
	
}

// Permutation without dynamic  pivoting 
void permutation1(Matrix &A, int* P, Matrix &A_new)
{
	int n = A.row_num;
	int nz = A.nnz;
	A_new.row_num = n;
	A_new.col_num = n;
	A_new.nnz = nz;

	int* Pinv = new int[n];
	for (int i = 0; i < n; ++i)
	{
		Pinv[P[i]] = i;
	}

	int jnew = 0, inew = 0;
	A_new.col_ptr.push_back(0);

	for (jnew = 0; jnew < n; ++jnew)
	{
		int j = P[jnew];
		A_new.col_ptr.push_back(A_new.col_ptr[jnew] + A.col_ptr[j+1] - A.col_ptr[j]);
		vector<Entry> tmpcol;
		for (int i = A.col_ptr[j]; i < A.col_ptr[j+1]; ++i)
		{
			inew = Pinv[A.row_idx[i]];
			Entry tmp(inew, A.x[i]);
			tmpcol.push_back(tmp);
		}
		sort(tmpcol.begin(), tmpcol.end());
		for (int i = 0; i < tmpcol.size(); ++i)
		{
			A_new.row_idx.push_back(tmpcol[i].row_idx);
			A_new.x.push_back(tmpcol[i].val);
		}
	}
	
}
