#include <iostream>
#include <cstdlib>
#include <vector>
#include <set>
#include <algorithm>
#include <iterator>
#include "matrix.h"
#include "Timer.h"
#include "symbolic.h"
#include <list>
#include <cmath>

using namespace std;

// symbolic phase
void Symbolic_Matrix :: symbolic(Matrix &A)
{
	int nz = A.nnz;
	n = A.row_num;
	sym_c_ptr.push_back(0);
	vector<int> :: iterator colhead;
	vector<int> :: iterator colend;
	vector<int> :: iterator it;
	
	// process columns from 0 to n-1
	for (int i = 0; i < n; ++i)
	{
		colhead = A.row_idx.begin() + A.col_ptr[i];
		colend = A.row_idx.begin() + A.col_ptr[i+1];

		vector<int> tmpcol1;
		vector<int> tmpcol2(colhead, colend);
		sort(tmpcol2.begin(), tmpcol2.end());

		//search depended columns in the left
		for (unsigned int j = 0; j < tmpcol2.size(); ++j)
		{
			int nz_idx = tmpcol2[j];
			if (nz_idx < i)
			{
				colhead = sym_r_idx.begin() + l_col_ptr[nz_idx];
				colend = sym_r_idx.begin() + sym_c_ptr[nz_idx+1];
				tmpcol1.resize(distance(colhead, colend) + tmpcol2.size());
				it = set_union(colhead, colend, tmpcol2.begin(), tmpcol2.end(), tmpcol1.begin());
				tmpcol1.resize(it - tmpcol1.begin());
				tmpcol2.clear();
				swap(tmpcol1, tmpcol2);
			}
			else
				break;
		}
		
		for (unsigned int j = 0; j < tmpcol2.size(); ++j)
		{
			if (tmpcol2[j] == i)
			{
				l_col_ptr.push_back(j + sym_c_ptr.back());
			}
		}
		sym_c_ptr.push_back(tmpcol2.size() + sym_c_ptr.back());
		sym_r_idx.insert(sym_r_idx.end(), tmpcol2.begin(), tmpcol2.end());
	}

	nnz = sym_c_ptr.back();

	cout << "Original nonzero: " << nz << endl;
	cout << "Symbolic nonzero: " << nnz << endl;
}

void Symbolic_Matrix :: csr()
{
	csr_r_ptr.push_back(0);	
	vector<vector<int> > buffer(n, vector<int>());

	for (int i = 0; i < n; ++i)
	{
		for (int j = sym_c_ptr[i]; j < sym_c_ptr[i+1]; ++j)
		{
			int row_idx = sym_r_idx[j];
			buffer[row_idx].push_back(i);
		}
	}
	
	for (int i = 0; i < n; ++i)
	{
		for (unsigned int j = 0; j < buffer[i].size(); ++j)
		{
			csr_c_idx.push_back(buffer[i][j]);
			if(buffer[i][j] == i)
				csr_diag_ptr.push_back(csr_c_idx.size()-1);
		}
		csr_r_ptr.push_back(csr_r_ptr.back() + buffer[i].size());
	}
}

void Symbolic_Matrix :: predictLU (Matrix &A)
{

	for (int i = 0; i < n; ++i)
	{
		vector<int> :: iterator start = A.row_idx.begin() + A.col_ptr[i];
		vector<int> :: iterator end = A.row_idx.begin() + A.col_ptr[i+1];
		for (int j = sym_c_ptr[i]; j < sym_c_ptr[i+1]; ++j)
		{
			vector<int> :: iterator idx = find(start, end, sym_r_idx[j]); 
			if (idx != end)
			{	
				vector<float> :: iterator x_idx = A.x.begin() + distance(A.row_idx.begin(), idx);
				val.push_back(*x_idx);
			}
			else
			{
				val.push_back(0);
			}
		}
	}
	val_bak.assign(val.begin(), val.end());
}

void Symbolic_Matrix :: restore()
{
	val = val_bak;
}

void Symbolic_Matrix :: U_dependency()
{
    for (int i = 0; i < n; ++i)
	{
		list<int> tmp;
		graph.push_back(tmp);
		// graph_inv.push_back(tmp);
	}

	for (int i = 0; i < n; ++i)
		if (l_col_ptr[i] != sym_c_ptr[i + 1] - 1)
			for(int k = csr_r_ptr[i] + 1; k < csr_r_ptr[i + 1]; ++k)
				if(csr_c_idx[k] > i)
				{
					graph[i].push_back(csr_c_idx[k]);
				}
}

void Symbolic_Matrix :: W_dependency()
{
	for (int i = 0; i < n; ++i)
	{
		if(sym_c_ptr[i + 1] - sym_c_ptr[i] > 2)
		{
			for(int m = sym_c_ptr[i]; m < l_col_ptr[i] - 1; m++)
			{
				int col1 = sym_r_idx[m];
				vector<int> :: iterator start1 = sym_r_idx.begin() + l_col_ptr[col1] + 1;
				vector<int> :: iterator end1 = sym_r_idx.begin() + sym_c_ptr[col1 + 1];
					
				for(int n = m + 1; n < l_col_ptr[i]; n++)
				{
					int flag_break = 0;
					int col2 = sym_r_idx[n];
					vector<int> :: iterator start2 = sym_r_idx.begin() + l_col_ptr[col2] + 1;
					vector<int> :: iterator end2 = sym_r_idx.begin() + sym_c_ptr[col2 + 1];
				
					for(int k = n + 1; k < sym_c_ptr[i + 1]; k++)
					{
						int col3 = sym_r_idx[k];
						if(binary_search(start1, end1, col2) && binary_search(start2, end2, col3))
						{
							graph[col1].push_back(col2);
							flag_break = 1;
							break;
						}
					}
					if(flag_break) break;
				}
			}					
		}
	}
}

void Symbolic_Matrix :: W_dependency_csr()
{
	for (int i = 0; i < n; ++i)
	{
		vector<int> line1(&csr_c_idx[csr_r_ptr[i]], &csr_c_idx[csr_r_ptr[i + 1]]);
		for (int t = l_col_ptr[i] + 1; t < sym_c_ptr[i + 1]; ++t)
		{
			for (int j = l_col_ptr[sym_r_idx[t]] + 1; j < sym_c_ptr[sym_r_idx[t] + 1]; ++j)
			{
				vector<int> line2(&csr_c_idx[csr_r_ptr[sym_r_idx[j]]], &csr_c_idx[csr_r_ptr[sym_r_idx[j] + 1]]);
				unsigned int n1 = 0;
				unsigned int n2 = 0;
				int flag_break = 0;
				while(n1 < line1.size() && n2 < line2.size())
				{
					if(line1[n1] <= i)
						n1++;
					if(line2[n2] <= sym_r_idx[t] || line2[n2] < line1[n1])
						n2++;
					else if(line2[n2] > line1[n1])
						n1++;
					else //line2[n2] == line1[n1]
					{
						flag_break = 1;
						graph[i].push_back(sym_r_idx[t]);
						// graph_inv[sym_r_idx[t]].push_back(i);
						break;
					}
				}
				if (flag_break)
					break;
			}
		}
	}
}

void Symbolic_Matrix :: W_dependency_csc()
{
	for (int i = 0; i < n; ++i)
	{
		if(sym_c_ptr[i + 1] - sym_c_ptr[i] > 2)
		{
			vector<int> line1(&sym_r_idx[sym_c_ptr[i]], &sym_r_idx[sym_c_ptr[i + 1]]);
			for(int m = sym_c_ptr[i]; m < l_col_ptr[i] - 1; m++)
			{
				int col1 = sym_r_idx[m];
				vector<int> line2(&sym_r_idx[l_col_ptr[col1] + 1], &sym_r_idx[sym_c_ptr[col1 + 1]]);
				unsigned int n2 = 0;
				unsigned int n1 = m - sym_c_ptr[i] + 1;

				while(n1 < line1.size() && n2 < line2.size())
				{
					if(line1[n1] < line2[n2])
						n1++;
					else if(line1[n1] > line2[n2])
						n2++;
					else
					{
						int col2 = line1[n1];
						vector<int> line3(&sym_r_idx[l_col_ptr[col2] + 1], &sym_r_idx[sym_c_ptr[col2 + 1]]);
						unsigned int n1_1 = n1 + 1;
						unsigned int n3 = 0;

						while(n1_1 < line1.size() && n3 < line3.size())
						{
							if(line1[n1_1] < line3[n3])
								n1_1++;
							else if(line1[n1] > line3[n3])
								n3++;
							else
							{
								graph[col1].push_back(col2);
								break;
							}
						}
						n1++;
					}
				}				
			}			
		}
	}
}

void Symbolic_Matrix :: remove_duplicates()
{
	for(int i = 0; i < n; ++i)
	{
		graph[i].sort();
		graph[i].unique();
	}
}

void Symbolic_Matrix :: BFS()
{
	level_idx.clear();
	level_size.clear();
	level_ptr.clear();
	num_lev = 0;

	graph_inv.clear();
    for (int i = 0; i < n; ++i)
	{
		list<int> tmp;
		graph_inv.push_back(tmp);
	}

	for(int i = 0; i < n; ++i)
		for(list<int>::iterator it = graph[i].begin(); it != graph[i].end(); ++it)
			graph_inv[*it].push_back(i);

	vector<int> count(n, 0);
	for(int i = 0; i < n; ++i)
	{
		count[i] = graph_inv[i].size();
	}

    list<int> S;
// First search for empty cols
	for (int i = 0; i < n; ++i)
		if(count[i] == 0)
			S.push_back(i);

	int total = 0;
	int num = 0;
	S.push_back(-1);

//BFS
    level_ptr.push_back(0);

    while(S.size() > 0) 
    {
    	if(S.front() == -1)
    	{
    		S.pop_front();
    		level_size.push_back(num);
			level_ptr.push_back(total);
			if(total == n)
    			break;
			num = 0;
			S.push_back(-1);
    	}
    	else
    	{
    		int i = S.front();
			S.pop_front();
			if(count[i] == 0)
			{
				level_idx.push_back(i);
				num ++;
				total ++;
				for(list<int>:: iterator it = graph[i].begin(); it != graph[i].end(); it++)
				{
					count[*it] --;
					if(count[*it] == 0)
						S.push_back(*it);
				}
			}
    	}
    }
	num_lev = level_size.size();
	cout << "Number of levels: " << num_lev << endl;
}

void Symbolic_Matrix :: restart()
{
	graph.clear();
	graph_inv.clear();
	level_idx.clear();
	level_size.clear();
	level_ptr.clear();
	num_lev = 0;
}

void Symbolic_Matrix :: U_leveling()
{
	restart();

	// t.start();
    U_dependency(); 
    // t.elapsedUserTime(utime);
    // t.elapsedWallclockTime(utime);  
    // cout << "U_dependency... " << utime << " ms" << endl;

	// t.start();
    BFS();
    // t.elapsedUserTime(utime);
    // t.elapsedWallclockTime(utime);
    // cout << "BFS... " << utime << " ms" << endl;
}

void Symbolic_Matrix :: U_W_leveling()
{
	restart();

	// t.start();
    U_dependency(); 
    // t.elapsedUserTime(utime);
    // t.elapsedWallclockTime(utime);  
    // cout << "U_dependency... " << utime << " ms" << endl;

    // t.start();
    W_dependency_csc();
    // t.elapsedUserTime(utime);
    // t.elapsedWallclockTime(utime); 
    // cout << "W_dependency... " << utime << " ms" << endl;

    // t.start();
    remove_duplicates();
    // t.elapsedUserTime(utime);
    // t.elapsedWallclockTime(utime); 
    // cout << "Remove_duplicates... " << utime << " ms" << endl;

	// t.start();
    BFS();
    // t.elapsedUserTime(utime);
    // t.elapsedWallclockTime(utime);
    // cout << "BFS... " << utime << " ms" << endl;
}
